#include <stdlib.h>
#include "envcube.h"
#include "nvmediapath/nvmediapath.h"
#include "nvtexture/nvtexture.h"
#include <GLES/gl.h>
#include "glerror.h"

void EnvCubeInitialize( EnvCube *cube )
{
    cube->status = 1;
}

void EnvCubeDestroy( EnvCube *cube )
{
}

static GLuint texids[4][6];
static int current = 2;

static void EnvCubeInitializeTextures()
{
    char filename[64];
    char pathname[128];
    char fullpathname[256];
    char *names[4] = { "nvlobby", "patio", "nvamphi", "nvwalkway" };
    char *faces[6] = { "right", "left", "top", "bottom", "back", "front" };
    int n;
    int f;

    for ( n = 0; n < 4; n++ )
    {
        glGenTextures( 6, texids[n] );
        for ( f = 0; f < 6; f++ )
        {
            glBindTexture( GL_TEXTURE_2D, texids[n][f] );
            sprintf( filename, "%s_%s.dds", names[n], faces[f] );
            sprintf( pathname, "MEDIA/textures/cubemaps/%s/", names[n] );
            LocateMediaFile( filename, pathname, fullpathname );
            if ( !LoadTextureDDS( fullpathname ) )
            {
                exit(-1);
            }
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
            glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
        }
    }

    CheckGLError( "EnvCubeBuildCube" );
}

EnvCube *EnvCubeBuildCube()
{
    EnvCube *cube;

    cube = (EnvCube*) calloc(1, sizeof(EnvCube));
    EnvCubeInitialize( cube );

    EnvCubeInitializeTextures();

    return cube;
}

void EnvCycleNextCubeMap( EnvCube *cube )
{
    current = (current+1) % 4;
}


static float cube_vertices[6][3*4] = {
    {   //   {3, 2, 7, 6}, // right
        1.0f,  1.0f, -1.0f, // r t f
        1.0f, -1.0f, -1.0f, // r b f
        1.0f,  1.0f,  1.0f, // r t b
        1.0f, -1.0f,  1.0f  // r b b
    },
    {   //   {4, 5, 0, 1}, // left
       -1.0f,  1.0f,  1.0f, // l t b
       -1.0f, -1.0f,  1.0f, // l b b
       -1.0f,  1.0f, -1.0f, // l t f
       -1.0f, -1.0f, -1.0f  // l b f
    },
    {   //   {4, 0, 7, 3}, // top
       -1.0f,  1.0f,  1.0f, // l t b
       -1.0f,  1.0f, -1.0f, // l t f
        1.0f,  1.0f,  1.0f, // r t b
        1.0f,  1.0f, -1.0f  // r t f   
    },
    {  //   {1, 5, 2, 6}, // bottom
       -1.0f, -1.0f, -1.0f, // l b f
       -1.0f, -1.0f,  1.0f, // l b b
        1.0f, -1.0f, -1.0f, // r b f
        1.0f, -1.0f,  1.0f  // r b b        
    },
    {  //  {7, 6, 4, 5}, // back
        1.0f,  1.0f,  1.0f, // r t b
        1.0f, -1.0f,  1.0f, // r b b
       -1.0f,  1.0f,  1.0f, // l t b
       -1.0f, -1.0f,  1.0f  // l b b        
    },
    {  //  {0, 1, 3, 2}, // front
       -1.0f,  1.0f, -1.0f, // l t f
       -1.0f, -1.0f, -1.0f, // l b f
        1.0f,  1.0f, -1.0f, // r t f
        1.0f, -1.0f, -1.0f // r b f
    }
};

static float cube_texcoords[6][2*4] = {
    {   //   {3, 2, 7, 6}, // right
       0.0f, 1.0f, // r t f
       0.0f, 0.0f, // r b f
       1.0f, 1.0f, // r t b
       1.0f, 0.0f  // r b b
    },
    {   //   {4, 5, 0, 1}, // left
       0.0f, 1.0f, // l t b
       0.0f, 0.0f, // l b b
       1.0f, 1.0f, // l t f
       1.0f, 0.0f  // l b f
    },
    {   //   {4, 0, 7, 3}, // top
       0.0f, 1.0f, // l t b
       0.0f, 0.0f, // l t f
       1.0f, 1.0f, // r t b
       1.0f, 0.0f  // r t f
    },
    {  //   {1, 5, 2, 6}, // bottom
       0.0f, 1.0f, // l b f
       0.0f, 0.0f, // l b b
       1.0f, 1.0f, // r b f
       1.0f, 0.0f  // r b b
    },
    {  //  {7, 6, 4, 5}, // back
       0.0f, 1.0f, // r t b
       0.0f, 0.0f, // r b b
       1.0f, 1.0f, // l t b
       1.0f, 0.0f  // l b b
    },
    {  //  {0, 1, 3, 2}, // front
       0.0f, 1.0f, // l t f
       0.0f, 0.0f, // l b f
       1.0f, 1.0f, // r t f
       1.0f, 0.0f  // r b f
    }
};

static GLbyte cube_bytetexcoords[6][2*4] = {
    {   //   {3, 2, 7, 6}, // right
         0, 127, // r t f
         0,   0, // r b f
       127, 127, // r t b
       127,   0  // r b b
    },
    {   //   {4, 5, 0, 1}, // left
         0, 127, // l t b
         0,   0, // l b b
       127, 127, // l t f
       127,   0  // l b f
    },
    {   //   {4, 0, 7, 3}, // top
         0, 127, // l t b
         0,   0, // l t f
         0, 127, // r t b
       127,   0  // r t f
    },
    {  //   {1, 5, 2, 6}, // bottom
         0, 127, // l b f
         0,   0, // l b b
       127, 127, // r b f
       127,   0  // r b b
    },
    {  //  {7, 6, 4, 5}, // back
         0, 127, // r t b
         0,   0, // r b b
       127, 127, // l t b
       127,   0  // l b b
    },
    {  //  {0, 1, 3, 2}, // front
         0, 127, // l t f
         0,   0, // l b f
       127, 127, // r t f
       127,   0  // r b f
    }
};

void EnvCubeDraw( EnvCube *cube )
{
    int i;

    glColor4f( 1.f, 1.f, 1.f, 0.f );

    glEnableClientState( GL_VERTEX_ARRAY );
    glEnableClientState( GL_TEXTURE_COORD_ARRAY );
    glEnable( GL_TEXTURE_2D );

    for ( i = 0; i < 6; i++ )
    {
        glVertexPointer( 3, GL_FLOAT, 0, cube_vertices[i] );
        glTexCoordPointer( 2, GL_FLOAT, 0, cube_texcoords[i] );
        //glTexCoordPointer( 2, GL_BYTE, 0, cube_bytetexcoords[i] );
        glBindTexture( GL_TEXTURE_2D, texids[current][i] );
        glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
    }

    glDisable( GL_TEXTURE_2D );
    glDisableClientState( GL_TEXTURE_COORD_ARRAY );
    glDisableClientState( GL_VERTEX_ARRAY );

    CheckGLError( "EnvCubeDraw" );
}
