#include "inverse.h"
#include "nvfxmath/nvfxmath.h"

static void identity( NvSFx m[4][4] )
{
    /*
    m[0][0] = 1.0f; m[0][1] = 0.0f; m[0][2] = 0.0f; m[0][3] = 0.0f;
    m[1][0] = 0.0f; m[1][1] = 1.0f; m[1][2] = 0.0f; m[1][3] = 0.0f;
    m[2][0] = 0.0f; m[2][1] = 0.0f; m[2][2] = 1.0f; m[2][3] = 0.0f;
    m[3][0] = 0.0f; m[3][1] = 0.0f; m[3][2] = 0.0f; m[3][3] = 1.0f;
    */

    m[0][0] = 0x00010000; m[0][1] = 0x00000000; m[0][2] = 0x00000000; m[0][3] = 0x00000000;
    m[1][0] = 0x00000000; m[1][1] = 0x00010000; m[1][2] = 0x00000000; m[1][3] = 0x00000000;
    m[2][0] = 0x00000000; m[2][1] = 0x00000000; m[2][2] = 0x00010000; m[2][3] = 0x00000000;
    m[3][0] = 0x00000000; m[3][1] = 0x00000000; m[3][2] = 0x00000000; m[3][3] = 0x00010000;
}

void inverse( NvSFx minv[4][4], NvSFx from[4][4] )
{
  NvSFx wtemp[4][8];
  NvSFx m0,m1,m2,m3,s;
  NvSFx *r0,*r1,*r2,*r3, *rtemp;

  /*
  r0 = wtemp[0];
  r1 = wtemp[1];
  r2 = wtemp[2];
  r3 = wtemp[3];
  r0[0] = from[0][0];             // build up [A][I]
  r0[1] = from[0][1];
  r0[2] = from[0][2];
  r0[3] = from[0][3];
  r0[4] = 1.0;
  r0[5] = 0.0;
  r0[6] = 0.0;
  r0[7] = 0.0;
  r1[0] = from[1][0];
  r1[1] = from[1][1];
  r1[2] = from[1][2];
  r1[3] = from[1][3];
  r1[4] = 0.0;
  r1[5] = 1.0;
  r1[6] = 0.0;
  r1[7] = 0.0;
  r2[0] = from[2][0];
  r2[1] = from[2][1];
  r2[2] = from[2][2];
  r2[3] = from[2][3];
  r2[4] = 0.0;
  r2[5] = 0.0;
  r2[6] = 1.0;
  r2[7] = 0.0;
  r3[0] = from[3][0];
  r3[1] = from[3][1];
  r3[2] = from[3][2];
  r3[3] = from[3][3];
  r3[4] = 0.0;
  r3[5] = 0.0;
  r3[6] = 0.0;
  r3[7] = 1.0;

  if (r0[0] == 0.0) {             // swap rows if needed
    if (r1[0] == 0.0) {
      if (r2[0] == 0.0) {
        if (r3[0] == 0.0) // singular
        {
            identity( minv );
            return;
        }
        rtemp = r0;
        r0 = r3;
        r3 = rtemp;
      } else {
        rtemp = r0;
        r0 = r2;
        r2 = rtemp;
      }
    } else {
      rtemp = r0;
      r0 = r1;
      r1 = rtemp;
    }
  }

  m1 = r1[0]/r0[0];               // eliminate first variable
  m2 = r2[0]/r0[0];
  m3 = r3[0]/r0[0];
  s = r0[1];
  r1[1] = r1[1] - m1 * s;
  r2[1] = r2[1] - m2 * s;
  r3[1] = r3[1] - m3 * s;
  s = r0[2];
  r1[2] = r1[2] - m1 * s;
  r2[2] = r2[2] - m2 * s;
  r3[2] = r3[2] - m3 * s;
  s = r0[3];
  r1[3] = r1[3] - m1 * s;
  r2[3] = r2[3] - m2 * s;
  r3[3] = r3[3] - m3 * s;
  s = r0[4];
  if (s != 0.0) {
    r1[4] = r1[4] - m1 * s;
    r2[4] = r2[4] - m2 * s;
    r3[4] = r3[4] - m3 * s;
  }
  s = r0[5];
  if (s != 0.0) {
    r1[5] = r1[5] - m1 * s;
    r2[5] = r2[5] - m2 * s;
    r3[5] = r3[5] - m3 * s;
  }
  s = r0[6];
  if (s != 0.0) {
    r1[6] = r1[6] - m1 * s;
    r2[6] = r2[6] - m2 * s;
    r3[6] = r3[6] - m3 * s;
  }
  s = r0[7];
  if (s != 0.0) {
    r1[7] = r1[7] - m1 * s;
    r2[7] = r2[7] - m2 * s;
    r3[7] = r3[7] - m3 * s;
  }

  if (r1[1] == 0.0) {             // swap rows if needed
    if (r2[1] == 0.0) {
      if (r3[1] == 0.0) // singular
      {
          identity( minv );
          return;
      }
      rtemp = r1;
      r1 = r3;
      r3 = rtemp;
    } else {
      rtemp = r1;
      r1 = r2;
      r2 = rtemp;
    }
  }
  m2 = r2[1]/r1[1];               // eliminate second variable
  m3 = r3[1]/r1[1];
  r2[2] = r2[2] - m2 * r1[2];
  r3[2] = r3[2] - m3 * r1[2];
  r3[3] = r3[3] - m3 * r1[3];
  r2[3] = r2[3] - m2 * r1[3];
  s = r1[4];
  if (s != 0.0) {
    r2[4] = r2[4] - m2 * s;
    r3[4] = r3[4] - m3 * s;
  }
  s = r1[5];
  if (s != 0.0) {
    r2[5] = r2[5] - m2 * s;
    r3[5] = r3[5] - m3 * s;
  }
  s = r1[6];
  if (s != 0.0) {
    r2[6] = r2[6] - m2 * s;
    r3[6] = r3[6] - m3 * s;
  }
  s = r1[7];
  if (s != 0.0) {
    r2[7] = r2[7] - m2 * s;
    r3[7] = r3[7] - m3 * s;
  }

  if (r2[2] == 0.0) {             // swap last 2 rows if needed
    if (r3[2] == 0.0) // singular
    {
        identity( minv );
        return;
    }
    rtemp = r2;
    r2 = r3;
    r3 = rtemp;
  }
  m3 = r3[2]/r2[2];               // eliminate third variable
  r3[3] = r3[3] - m3 * r2[3];
  r3[4] = r3[4] - m3 * r2[4];
  r3[5] = r3[5] - m3 * r2[5];
  r3[6] = r3[6] - m3 * r2[6];
  r3[7] = r3[7] - m3 * r2[7];

  if (r3[3] == 0.0) // singular
  {
      identity( minv );
      return;
  }

  s = 1.0f/r3[3];          // now back substitute row 3
  r3[4] = r3[4] * s;
  r3[5] = r3[5] * s;
  r3[6] = r3[6] * s;
  r3[7] = r3[7] * s;

  m2 = r2[3];                     // now back substitute row 2
  s = 1.0f/r2[2];
  r2[4] = s * (r2[4] - r3[4] * m2);
  r2[5] = s * (r2[5] - r3[5] * m2);
  r2[6] = s * (r2[6] - r3[6] * m2);
  r2[7] = s * (r2[7] - r3[7] * m2);
  m1 = r1[3];
  r1[4] = (r1[4] - r3[4] * m1);
  r1[5] = (r1[5] - r3[5] * m1);
  r1[6] = (r1[6] - r3[6] * m1);
  r1[7] = (r1[7] - r3[7] * m1);
  m0 = r0[3];
  r0[4] = (r0[4] - r3[4] * m0);
  r0[5] = (r0[5] - r3[5] * m0);
  r0[6] = (r0[6] - r3[6] * m0);
  r0[7] = (r0[7] - r3[7] * m0);

  m1 = r1[2];                     // now back substitute row 1
  s = 1.0f/r1[1];
  r1[4] = s * (r1[4] - r2[4] * m1);
  r1[5] = s * (r1[5] - r2[5] * m1);
  r1[6] = s * (r1[6] - r2[6] * m1);
  r1[7] = s * (r1[7] - r2[7] * m1);
  m0 = r0[2];
  r0[4] = (r0[4] - r2[4] * m0);
  r0[5] = (r0[5] - r2[5] * m0);
  r0[6] = (r0[6] - r2[6] * m0);
  r0[7] = (r0[7] - r2[7] * m0);

  m0 = r0[1];                     // now back substitute row 0
  s = 1.0f/r0[0];
  r0[4] = s * (r0[4] - r1[4] * m0);
  r0[5] = s * (r0[5] - r1[5] * m0);
  r0[6] = s * (r0[6] - r1[6] * m0);
  r0[7] = s * (r0[7] - r1[7] * m0);

  minv[0][0] = r0[4];  minv[0][1] = r0[5];  minv[0][2] = r0[6];  minv[0][3] = r0[7];
  minv[1][0] = r1[4];  minv[1][1] = r1[5];  minv[1][2] = r1[6];  minv[1][3] = r1[7];
  minv[2][0] = r2[4];  minv[2][1] = r2[5];  minv[2][2] = r2[6];  minv[2][3] = r2[7];
  minv[3][0] = r3[4];  minv[3][1] = r3[5];  minv[3][2] = r3[6];  minv[3][3] = r3[7];
  */

  r0 = wtemp[0];
  r1 = wtemp[1];
  r2 = wtemp[2];
  r3 = wtemp[3];
  r0[0] = from[0][0];             /* build up [A][I]      */
  r0[1] = from[0][1];
  r0[2] = from[0][2];
  r0[3] = from[0][3];
  r0[4] = 0x00010000;
  r0[5] = 0x00000000;
  r0[6] = 0x00000000;
  r0[7] = 0x00000000;
  r1[0] = from[1][0];
  r1[1] = from[1][1];
  r1[2] = from[1][2];
  r1[3] = from[1][3];
  r1[4] = 0x00000000;
  r1[5] = 0x00010000;
  r1[6] = 0x00000000;
  r1[7] = 0x00000000;
  r2[0] = from[2][0];
  r2[1] = from[2][1];
  r2[2] = from[2][2];
  r2[3] = from[2][3];
  r2[4] = 0x00000000;
  r2[5] = 0x00000000;
  r2[6] = 0x00010000;
  r2[7] = 0x00000000;
  r3[0] = from[3][0];
  r3[1] = from[3][1];
  r3[2] = from[3][2];
  r3[3] = from[3][3];
  r3[4] = 0x00000000;
  r3[5] = 0x00000000;
  r3[6] = 0x00000000;
  r3[7] = 0x00010000;

  if (r0[0] == 0x00000000) {     /* swap rows if needed          */
    if (r1[0] == 0x00000000) {
      if (r2[0] == 0x00000000) {
        if (r3[0] == 0x00000000) /* singular */
        {
            identity( minv );
            return;
        }
        rtemp = r0;
        r0 = r3;
        r3 = rtemp;
      } else {
        rtemp = r0;
        r0 = r2;
        r2 = rtemp;
      }
    } else {
      rtemp = r0;
      r0 = r1;
      r1 = rtemp;
    }
  }

  m1 = NvSFxDiv( r1[0], r0[0] ); /* eliminate first variable */
  m2 = NvSFxDiv( r2[0], r0[0] );
  m3 = NvSFxDiv( r3[0], r0[0] );
  s = r0[1];
  r1[1] = NvSFxSub( r1[1], NvSFxMul( m1, s ) );
  r2[1] = NvSFxSub( r2[1], NvSFxMul( m2, s ) );
  r3[1] = NvSFxSub( r3[1], NvSFxMul( m3, s ) );
  s = r0[2];
  r1[2] = NvSFxSub( r1[2], NvSFxMul( m1, s ) );
  r2[2] = NvSFxSub( r2[2], NvSFxMul( m2, s ) );
  r3[2] = NvSFxSub( r3[2], NvSFxMul( m3, s ) );
  s = r0[3];
  r1[3] = NvSFxSub( r1[3], NvSFxMul( m1, s ) );
  r2[3] = NvSFxSub( r2[3], NvSFxMul( m2, s ) );
  r3[3] = NvSFxSub( r3[3], NvSFxMul( m3, s ) );
  s = r0[4];
  if (s != 0x00000000) {
    r1[4] = NvSFxSub( r1[4], NvSFxMul( m1, s ) );
    r2[4] = NvSFxSub( r2[4], NvSFxMul( m2, s ) );
    r3[4] = NvSFxSub( r3[4], NvSFxMul( m3, s ) );
  }
  s = r0[5];
  if (s != 0x00000000) {
    r1[5] = NvSFxSub( r1[5], NvSFxMul( m1, s ) );
    r2[5] = NvSFxSub( r2[5], NvSFxMul( m2, s ) );
    r3[5] = NvSFxSub( r3[5], NvSFxMul( m3, s ) );
  }
  s = r0[6];
  if (s != 0x00000000) {
    r1[6] = NvSFxSub( r1[6], NvSFxMul( m1, s ) );
    r2[6] = NvSFxSub( r2[6], NvSFxMul( m2, s ) );
    r3[6] = NvSFxSub( r3[6], NvSFxMul( m3, s ) );
  }
  s = r0[7];
  if (s != 0x00000000) {
    r1[7] = NvSFxSub( r1[7], NvSFxMul( m1, s ) );
    r2[7] = NvSFxSub( r2[7], NvSFxMul( m2, s ) );
    r3[7] = NvSFxSub( r3[7], NvSFxMul( m3, s ) );
  }

  if (r1[1] == 0x00000000) {   /* swap rows if needed          */
    if (r2[1] == 0x00000000) {
      if (r3[1] == 0x00000000) /* singular */
      {
          identity( minv );
          return;
      }
      rtemp = r1;
      r1 = r3;
      r3 = rtemp;
    } else {
      rtemp = r1;
      r1 = r2;
      r2 = rtemp;
    }
  }
  m2 = NvSFxDiv( r2[1], r1[1] ); /* eliminate second variable    */
  m3 = NvSFxDiv( r3[1], r1[1] );
  r2[2] = NvSFxSub( r2[2], NvSFxMul( m2, r1[2] ) );
  r3[2] = NvSFxSub( r3[2], NvSFxMul( m3, r1[2] ) );
  r3[3] = NvSFxSub( r3[3], NvSFxMul( m3, r1[3] ) );
  r2[3] = NvSFxSub( r2[3], NvSFxMul( m2, r1[3] ) );
  s = r1[4];
  if (s != 0x00000000) {
    r2[4] = NvSFxSub( r2[4], NvSFxMul( m2, s ) );
    r3[4] = NvSFxSub( r3[4], NvSFxMul( m3, s ) );
  }
  s = r1[5];
  if (s != 0x00000000) {
    r2[5] = NvSFxSub( r2[5], NvSFxMul( m2, s ) );
    r3[5] = NvSFxSub( r3[5], NvSFxMul( m3, s ) );
  }
  s = r1[6];
  if (s != 0x00000000) {
    r2[6] = NvSFxSub( r2[6], NvSFxMul( m2, s ) );
    r3[6] = NvSFxSub( r3[6], NvSFxMul( m3, s ) );
  }
  s = r1[7];
  if (s != 0x00000000) {
    r2[7] = NvSFxSub( r2[7], NvSFxMul( m2, s ) );
    r3[7] = NvSFxSub( r3[7], NvSFxMul( m3, s ) );
  }

  if (r2[2] == 0x00000000) { /* swap last 2 rows if needed   */
    if (r3[2] == 0x00000000) /* singular */
    {
        identity( minv );
        return;
    }
    rtemp = r2;
    r2 = r3;
    r3 = rtemp;
  }
  m3 = NvSFxDiv( r3[2], r2[2] ); /* eliminate third variable     */
  r3[3] = NvSFxSub( r3[3], NvSFxMul( m3, r2[3] ) );
  r3[4] = NvSFxSub( r3[4], NvSFxMul( m3, r2[4] ) );
  r3[5] = NvSFxSub( r3[5], NvSFxMul( m3, r2[5] ) );
  r3[6] = NvSFxSub( r3[6], NvSFxMul( m3, r2[6] ) );
  r3[7] = NvSFxSub( r3[7], NvSFxMul( m3, r2[7] ) );

  if (r3[3] == 0x00000000) /* singular */
  {
      identity( minv );
      return;
  }

  s = NvSFxRecip( r3[3] );   /* now back substitute row 3    */
  r3[4] = NvSFxMul( r3[4], s );
  r3[5] = NvSFxMul( r3[5], s );
  r3[6] = NvSFxMul( r3[6], s );
  r3[7] = NvSFxMul( r3[7], s );

  m2 = r2[3];                /* now back substitute row 2    */
  s = NvSFxRecip( r2[2] );
  r2[4] = NvSFxMul( s, NvSFxSub( r2[4], NvSFxMul( r3[4], m2 ) ) );
  r2[5] = NvSFxMul( s, NvSFxSub( r2[5], NvSFxMul( r3[5], m2 ) ) );
  r2[6] = NvSFxMul( s, NvSFxSub( r2[6], NvSFxMul( r3[6], m2 ) ) );
  r2[7] = NvSFxMul( s, NvSFxSub( r2[7], NvSFxMul( r3[7], m2 ) ) );
  m1 = r1[3];
  r1[4] = NvSFxSub( r1[4], NvSFxMul( r3[4], m1 ) );
  r1[5] = NvSFxSub( r1[5], NvSFxMul( r3[5], m1 ) );
  r1[6] = NvSFxSub( r1[6], NvSFxMul( r3[6], m1 ) );
  r1[7] = NvSFxSub( r1[7], NvSFxMul( r3[7], m1 ) );
  m0 = r0[3];
  r0[4] = NvSFxSub( r0[4], NvSFxMul( r3[4], m0 ) );
  r0[5] = NvSFxSub( r0[5], NvSFxMul( r3[5], m0 ) );
  r0[6] = NvSFxSub( r0[6], NvSFxMul( r3[6], m0 ) );
  r0[7] = NvSFxSub( r0[7], NvSFxMul( r3[7], m0 ) );

  m1 = r1[2];                /* now back substitute row 1    */
  s = NvSFxRecip( r1[1] );
  r1[4] = NvSFxMul( s, NvSFxSub( r1[4], NvSFxMul( r2[4], m1 ) ) );
  r1[5] = NvSFxMul( s, NvSFxSub( r1[5], NvSFxMul( r2[5], m1 ) ) );
  r1[6] = NvSFxMul( s, NvSFxSub( r1[6], NvSFxMul( r2[6], m1 ) ) );
  r1[7] = NvSFxMul( s, NvSFxSub( r1[7], NvSFxMul( r2[7], m1 ) ) );
  m0 = r0[2];
  r0[4] = NvSFxSub( r0[4], NvSFxMul( r2[4], m0 ) );
  r0[5] = NvSFxSub( r0[5], NvSFxMul( r2[5], m0 ) );
  r0[6] = NvSFxSub( r0[6], NvSFxMul( r2[6], m0 ) );
  r0[7] = NvSFxSub( r0[7], NvSFxMul( r2[7], m0 ) );

  m0 = r0[1];                /* now back substitute row 0    */
  s = NvSFxRecip( r0[0] );
  r0[4] = NvSFxMul( s, NvSFxSub( r0[4], NvSFxMul( r1[4], m0 ) ) );
  r0[5] = NvSFxMul( s, NvSFxSub( r0[5], NvSFxMul( r1[5], m0 ) ) );
  r0[6] = NvSFxMul( s, NvSFxSub( r0[6], NvSFxMul( r1[6], m0 ) ) );
  r0[7] = NvSFxMul( s, NvSFxSub( r0[7], NvSFxMul( r1[7], m0 ) ) );

  minv[0][0] = r0[4];  minv[0][1] = r0[5];  minv[0][2] = r0[6];  minv[0][3] = r0[7];
  minv[1][0] = r1[4];  minv[1][1] = r1[5];  minv[1][2] = r1[6];  minv[1][3] = r1[7];
  minv[2][0] = r2[4];  minv[2][1] = r2[5];  minv[2][2] = r2[6];  minv[2][3] = r2[7];
  minv[3][0] = r3[4];  minv[3][1] = r3[5];  minv[3][2] = r3[6];  minv[3][3] = r3[7];
}

