#include "quat.h"
#include <stdio.h>

#define M_PI 3.14159265358979323846

void MakeQuat( NvSFx quat[4], NvSFx radians, NvSFx axis[3] )
{
    /*
    fix dst_l = (fix) sin(float(radians)/2.0); // should be
    fix src_l = (fix) sqrt(float(axis[0]*axis[0]+axis[1]*axis[1]+axis[2]*axis[2]));   // actually is
    */

    NvSFx dst_l;
    NvSFx src_l;
    NvSFx degrees;
    /*
    degrees = (180.0/M_PI) * radians/2.0;
    */
    degrees = NvSFxMul( 0x00394BB8, NvSFxMul( 0x00008000, radians ) );
    dst_l = NvSFxSinDFullRange( degrees ); /* should be */
    src_l = NvSFxAdd(
                  NvSFxMul( axis[0], axis[0] ),
                  NvSFxAdd( NvSFxMul( axis[1], axis[1] ),
                            NvSFxMul( axis[2], axis[2] )
                  )
              );
    src_l = NvSFxSqrt( src_l ); /* actually is */

    /*
    if (src_l==0.0) {
      quat[0] = 0.0;
      quat[1] = 0.0;
      quat[2] = 0.0;
      quat[3] = 1.0;
    } else {
      fix s = dst_l / src_l;
      quat[0] = axis[0] * s;
      quat[1] = axis[1] * s;
      quat[2] = axis[2] * s;
      quat[3] = (fix) cos(float(radians)/2.0);
    }
    */
    if ( src_l == 0x00000000 )
    {
        quat[0] = 0x00000000;
        quat[1] = 0x00000000;
        quat[2] = 0x00000000;
        quat[3] = 0x00010000;
    }
    else
    {
        NvSFx s;
        s = NvSFxDiv( dst_l, src_l );
        quat[0] = NvSFxMul( axis[0], s );
        quat[1] = NvSFxMul( axis[1], s );
        quat[2] = NvSFxMul( axis[2], s );
        quat[3] = NvSFxCosDFullRange( degrees );
    }
}

void MulQuat( NvSFx c[4], NvSFx a[4], NvSFx b[4] )
{
    /*
    c[3] = a[3]*b[3] - a[0]*b[0] - a[1]*b[1] - a[2]*b[2];
    c[0] = a[1]*b[2] - b[1]*a[2] + a[3]*b[0] + b[3]*a[0];
    c[1] = a[2]*b[0] - b[2]*a[0] + a[3]*b[1] + b[3]*a[1];
    c[2] = a[0]*b[1] - b[0]*a[1] + a[3]*b[2] + b[3]*a[2];
    */

    c[3] = NvSFxSub(
                 NvSFxSub( NvSFxMul(a[3],b[3]), NvSFxMul(a[0],b[0]) ),
                 NvSFxAdd( NvSFxMul(a[1],b[1]), NvSFxMul(a[2],b[2]) )
             );
    c[0] = NvSFxAdd(
                 NvSFxSub( NvSFxMul(a[1],b[2]), NvSFxMul(b[1],a[2]) ),
                 NvSFxAdd( NvSFxMul(a[3],b[0]), NvSFxMul(b[3],a[0]) )
             );
    c[1] = NvSFxAdd(
                 NvSFxSub( NvSFxMul(a[2],b[0]), NvSFxMul(b[2],a[0]) ),
                 NvSFxAdd( NvSFxMul(a[3],b[1]), NvSFxMul(b[3],a[1]) )
             );
    c[2] = NvSFxAdd(
                 NvSFxSub( NvSFxMul(a[0],b[1]), NvSFxMul(b[0],a[1]) ),
                 NvSFxAdd( NvSFxMul(a[3],b[2]), NvSFxMul(b[3],a[2]) )
             );
}

void QuatToMat( NvSFx mat[4][4], NvSFx quat[4] )
{
    /*
    mat[0][0] = 1.0f + 2.0f * (-quat[1]*quat[1] - quat[2]*quat[2]);
	mat[0][1] = 2.0f * ( quat[0]*quat[1] + quat[3]*quat[2]);
	mat[0][2] = 2.0f * ( quat[0]*quat[2] - quat[3]*quat[1]);
	mat[0][3] = 0.0f;

	mat[1][0] = 2.0f * ( quat[0]*quat[1] - quat[3]*quat[2]);
	mat[1][1] = 1.0f + 2.0f * (-quat[0]*quat[0] - quat[2]*quat[2]);
	mat[1][2] = 2.0f * ( quat[1]*quat[2] + quat[3]*quat[0]);
	mat[1][3] = 0.0f;

	mat[2][0] = 2.0f * ( quat[0]*quat[2] + quat[3]*quat[1]);
	mat[2][1] = 2.0f * ( quat[1]*quat[2] - quat[3]*quat[0]);
	mat[2][2] = 1.0f + 2.0f * (-quat[0]*quat[0] - quat[1]*quat[1]);
	mat[2][3] = 0.0f;

	mat[3][0] = 0.0f;
    mat[3][1] = 0.0f;
    mat[3][2] = 0.0f;
    mat[3][3] = 1.0f;
    */

    mat[0][0] = NvSFxSub( 0x00010000, NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[1],quat[1]), NvSFxMul(quat[2],quat[2]) ) ) );
    mat[0][1] = NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[0],quat[1]), NvSFxMul(quat[3],quat[2]) ) );
	mat[0][2] = NvSFxMul( 0x00020000, NvSFxSub( NvSFxMul(quat[0],quat[2]), NvSFxMul(quat[3],quat[1]) ) );
	mat[0][3] = 0x00000000;

	mat[1][0] = NvSFxMul( 0x00020000, NvSFxSub( NvSFxMul(quat[0],quat[1]), NvSFxMul(quat[3],quat[2]) ) );
    mat[1][1] = NvSFxSub( 0x00010000, NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[0],quat[0]), NvSFxMul(quat[2],quat[2]) ) ) );
	mat[1][2] = NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[1],quat[2]), NvSFxMul(quat[3],quat[0]) ) );
	mat[1][3] = 0x00000000;

	mat[2][0] = NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[0],quat[2]), NvSFxMul(quat[3],quat[1]) ) );
	mat[2][1] = NvSFxMul( 0x00020000, NvSFxSub( NvSFxMul(quat[1],quat[2]), NvSFxMul(quat[3],quat[0]) ) );
    mat[2][2] = NvSFxSub( 0x00010000, NvSFxMul( 0x00020000, NvSFxAdd( NvSFxMul(quat[0],quat[0]), NvSFxMul(quat[1],quat[1]) ) ) );
	mat[2][3] = 0x00000000;

	mat[3][0] = 0x00000000;
    mat[3][1] = 0x00000000;
    mat[3][2] = 0x00000000;
    mat[3][3] = 0x00010000;
}

