#ifndef _SHAPE_H
#define _SHAPE_H

#include <stdio.h>
#include <stdlib.h>
#include "nvfxmath/nvfxmath.h"

#define MAX_DEPTH 64

struct _Vertex {

  NvSFx p[3];      /* point  */
  NvSFx n[3];      /* normal */

  NvSFx tcmapA[2]; /* texture coordinates for paraboloid map A */
  NvSFx tcmapB[2]; /* texture coordinates for paraboloid map B */
  NvSFx alpha;     /* alpha value used to select between texture maps */

  NvSFx v[3];      /* velocity      */
  NvSFx h[3];      /* home position */
  NvSFx a[3];      /* neighborhood velocity for averaging */

  int connectedness;    /* indicates how many triangles this vertex   */
                        /* is connected to, as well as how many edges */    
};

typedef struct _Vertex Vertex;

struct _Edge {
    short v0id;
    short v1id;
    NvSFx l; /* length */
};

typedef struct _Edge Edge;

struct _Tristrip {
  int numVerts;  /* numTris == numVerts-2 */
  Vertex **vertices;
  unsigned short *indices;
};

typedef struct _Tristrip Tristrip;

struct _Shape {
  int numTristrips;
  Tristrip *tristrips;

  int numEdges;
  Edge *edges;

  int numVerts;
  Vertex *vertices;

  NvSFx final_drag;
  NvSFx initial_drag;
  NvSFx drag;
};

typedef struct _Shape Shape;


void ShapeInitialize( Shape *shape );
void ShapeDestroy( Shape *shape );
void ShapeCycleNextCubeMap( Shape *shape );
void ShapeDraw( Shape *shape );
void ShapeCalcNormals( Shape *shape );
void ShapeCalcNormals( Shape *shape );
void CalcTexcoordsFromNormal( Shape *shape, NvSFx eye[3] );
void ShapeDump( Shape *shape );
  /* e is the eye point, n is the line of sight, t is the test point */
  /* returns distance squared                                        */
NvSFx ShapePickDistance( NvSFx e[3], NvSFx n[3], NvSFx t[3] );
void ShapePick( Shape *shape, NvSFx e[3], NvSFx n[3] );
void ShapeCalcVelocity( Shape *shape );
void ShapeFilterVelocity( Shape *shape );
void ShapeResetDrag( Shape *shape );
void ShapeDrawVertices( Shape *shape );
void ShapeDrawEdges( Shape *shape );
void ShapeDrawSphereMapTexture();
Shape *ShapeBuildSphere( int depth );

#endif

