/* 
   Copyright 1998-2003, NVIDIA Corporation.
   All rights Reserved.

   THE INFORMATION CONTAINED HEREIN IS PROPRIETARY AND CONFIDENTIAL TO
   NVIDIA, CORPORATION.  USE, REPRODUCTION OR DISCLOSURE TO ANY THIRD PARTY
   IS SUBJECT TO WRITTEN PRE-APPROVAL BY NVIDIA, CORPORATION.
*/

#include <termios.h>
#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <GLES/gl.h>

static struct termios save_termios;
static GLboolean opened;

//------------------------------------------------------------------------

int initRawKeyboard(void)
{
    struct termios buf;

    if (!opened) {
        if (tcgetattr(0, &save_termios) < 0) return -1;
        opened = GL_TRUE;
    }

    buf = save_termios;

    buf.c_lflag &= ~(ECHO | ICANON);

    buf.c_cc[VMIN] = 0;
    buf.c_cc[VTIME] = 0;

    return (tcsetattr(0, TCSAFLUSH, &buf) < 0) ? -1 : 0;
}

//Return 1 if data, 0 if no data (char pointer not updated)
//Return < 0 if error
int readRawKeyboard(unsigned char *data)
{
    ssize_t s;

    errno = 0;
    s = read(0, data, 1);

    return s ? s : errno ? -1 : 0;
}

//Restore blocking, buffering
void closeRawKeyboard(void)
{
    if (opened) {
        tcsetattr(0, TCSAFLUSH, &save_termios);
        opened = GL_FALSE;
    }
}
