
function OnFinish(selProj, selObj)
{
	try
	{
		var strProjectPath = wizard.FindSymbol('PROJECT_PATH');
		var strProjectName = wizard.FindSymbol('PROJECT_NAME');

		selProj = CreateCustomProject(strProjectName, strProjectPath);
		AddConfig(selProj, strProjectName);
		AddFilters(selProj);

		var InfFile = CreateCustomInfFile();
		AddFilesToCustomProj(selProj, strProjectName, strProjectPath, InfFile);
		PchSettings(selProj);
		InfFile.Delete();

		selProj.Object.Save();
	}
	catch(e)
	{
		if (e.description.length != 0)
			SetErrorInfo(e);
		return e.number
	}
}

function CreateCustomProject(strProjectName, strProjectPath)
{
	try
	{
		var strProjTemplatePath = wizard.FindSymbol('PROJECT_TEMPLATE_PATH');
		var strProjTemplate = '';
		strProjTemplate = strProjTemplatePath + '\\default.vcproj';

		var Solution = dte.Solution;
		var strSolutionName = "";
		if (wizard.FindSymbol("CLOSE_SOLUTION"))
		{
			Solution.Close();
			strSolutionName = wizard.FindSymbol("VS_SOLUTION_NAME");
			if (strSolutionName.length)
			{
				var strSolutionPath = strProjectPath.substr(0, strProjectPath.length - strProjectName.length);
				Solution.Create(strSolutionPath, strSolutionName);
			}
		}

		var strProjectNameWithExt = '';
		strProjectNameWithExt = strProjectName + '.vcproj';

		var oTarget = wizard.FindSymbol("TARGET");
		var prj;
		if (wizard.FindSymbol("WIZARD_TYPE") == vsWizardAddSubProject)  // vsWizardAddSubProject
		{
			var prjItem = oTarget.AddFromTemplate(strProjTemplate, strProjectNameWithExt);
			prj = prjItem.SubProject;
		}
		else
		{
			prj = oTarget.AddFromTemplate(strProjTemplate, strProjectPath, strProjectNameWithExt);
		}
		return prj;
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilters(proj)
{
	try
	{
        // Add the folders to the project
        var srcFolder = proj.Object.AddFilter('Source Files');
        srcFolder.Filter = "cpp;cxx;c";
        
        var hdrFolder = proj.Object.AddFilter('Header Files');
        hdrFolder.Filter = "h;hpp";
	}
	catch(e)
	{
		throw e;
	}
}

function AddConfig(proj, strProjectName)
{
	try
	{
        // debug build configuration
        var config = proj.Object.Configurations('Debug');
        config.IntermediateDirectory = 'Debug';
        config.OutputDirectory = 'Debug';

        var CLTool = config.Tools('VCCLCompilerTool');
        CLTool.Optimization = optimizeOption.optimizeDisabled;
        CLTool.AdditionalIncludeDirectories = "../../../../inc, ../../../../LIBS/inc";
        CLTool.PreprocessorDefinitions="WIN32;_DEBUG;_WINDOWS";
        CLTool.MinimalRebuild = true;
        CLTool.BasicRuntimeChecks = basicRuntimeCheckOption.runtimeBasicCheckAll;
        CLTool.RuntimeLibrary = runtimeLibraryOption.rtMultiThreadedDebug;
        CLTool.UsePrecompiledHeader = pchOption.pchNone;
        CLTool.WarningLevel = warningLevelOption.warningLevel_3;
        CLTool.Detect64BitPortabilityProblems = true;
        CLTool.DebugInformationFormat = debugOption.debugEditAndContinue;

        var LinkTool = config.Tools('VCLinkerTool');
        
        LinkTool.AdditionalDependencies = "opengl32.lib libGLES_CM.lib egl.lib nvfxmath.lib";
        LinkTool.IgnoreDefaultLibraryNames = "LIBCMT";
            
        LinkTool.OutputFile = "../../bin/x86/debug/" + strProjectName + ".exe";
        LinkTool.LinkIncremental = linkIncrementalType.linkIncrementalYes;
        LinkTool.AdditionalLibraryDirectories = "../../../../LIBS/lib/x86/debug";
        LinkTool.GenerateDebugInformation = true;
        
        LinkTool.SubSystem = subSystemOption.subSystemConsole;

        // release build configuration        
        config = proj.Object.Configurations('Release');
        config.IntermediateDirectory = 'Release';
        config.OutputDirectory = 'Release';

        var CLTool = config.Tools('VCCLCompilerTool');
        CLTool.AdditionalIncludeDirectories = "../../../../inc, ../../../../LIBS/inc";
        CLTool.PreprocessorDefinitions="WIN32;NDEBUG;_WINDOWS";
        CLTool.RuntimeLibrary = runtimeLibraryOption.rtMultiThreaded;
        CLTool.UsePrecompiledHeader = pchOption.pchNone;
        CLTool.Detect64BitPortabilityProblems = true;
        CLTool.WarningLevel = warningLevelOption.warningLevel_3;
        
        var LinkTool = config.Tools('VCLinkerTool');

        LinkTool.AdditionalDependencies = "opengl32.lib libGLES_CM.lib egl.lib nvfxmath.lib";
            
        LinkTool.OutputFile = "../../bin/x86/release/" + strProjectName + ".exe";
        LinkTool.LinkIncremental = linkIncrementalType.linkIncrementalNo;
        LinkTool.AdditionalLibraryDirectories = "../../../../LIBS/lib/x86/release";
        LinkTool.OptimizeReferences = optRefType.optReferences 

        LinkTool.SubSystem = subSystemOption.subSystemConsole;
	}
	catch(e)
	{
		throw e;
	}
}

function PchSettings(proj)
{
	// TODO: specify pch settings
}

function DelFile(fso, strWizTempFile)
{
	try
	{
		if (fso.FileExists(strWizTempFile))
		{
			var tmpFile = fso.GetFile(strWizTempFile);
			tmpFile.Delete();
		}
	}
	catch(e)
	{
		throw e;
	}
}

function CreateCustomInfFile()
{
	try
	{
		var fso, TemplatesFolder, TemplateFiles, strTemplate;
		fso = new ActiveXObject('Scripting.FileSystemObject');

		var TemporaryFolder = 2;
		var tfolder = fso.GetSpecialFolder(TemporaryFolder);
		var strTempFolder = tfolder.Drive + '\\' + tfolder.Name;

		var strWizTempFile = strTempFolder + "\\" + fso.GetTempName();

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');
		var strInfFile = strTemplatePath + '\\Templates.inf';
		wizard.RenderTemplate(strInfFile, strWizTempFile);

		var WizTempFile = fso.GetFile(strWizTempFile);
		return WizTempFile;
	}
	catch(e)
	{
		throw e;
	}
}

function GetTargetName(strName, strProjectName)
{
	try
	{
        var strTarget = strName;

        switch (strName)
        {
            case "Makefile.xscale":
                strTarget = 'Makefile.xscale';
                break;
                
            case "win32ui.c":
                strTarget = 'win32ui.c';
                break;
                
            case "simple.c":
                strTarget = strProjectName + ".c";
                break;

            default:
                break;
        }

		return strTarget; 
	}
	catch(e)
	{
		throw e;
	}
}

function AddFilesToCustomProj(proj, strProjectName, strProjectPath, InfFile)
{
	try
	{
		var projItems = proj.ProjectItems

		var strTemplatePath = wizard.FindSymbol('TEMPLATES_PATH');

		var strTpl = '';
		var strName = '';

		var strTextStream = InfFile.OpenAsTextStream(1, -2);
		while (!strTextStream.AtEndOfStream)
		{
			strTpl = strTextStream.ReadLine();
			if (strTpl != '')
			{
				strName = strTpl;
				var strTarget = GetTargetName(strName, strProjectName);
				var strTemplate = strTemplatePath + '\\' + strTpl;
				var strFile = strProjectPath + '\\' + strTarget;

				var bCopyOnly = false;  //"true" will only copy the file from strTemplate to strTarget without rendering/adding to the project
				var strExt = strName.substr(strName.lastIndexOf("."));
				if(strExt==".bmp" || strExt==".ico" || strExt==".gif" || strExt==".rtf" || strExt==".css")
					bCopyOnly = true;
				wizard.RenderTemplate(strTemplate, strFile, bCopyOnly);
				proj.Object.AddFile(strFile);
			}
		}
		strTextStream.Close();
	}
	catch(e)
	{
		throw e;
	}
}
